/*	NETMON config.c  (MITRE)	 12/18/88	*/

/*
 * config.c: configuration and other signalled routines for iptrace.
 *
 * Syntax of the configuration file:
 *
 * outfile	data|htm|sample	<filename>
 * nofile	data|htm|sample
 * set 		htminterval 	<mins>
 *
 * iptrace has dynamic reconfiguration to allow 2 types of function:
 *
 * summaries or samples from continuous monitoring
 * (e.g. host traffic matrix)
 *
 * complete data collection for performance measurement.
 *
 * To control both types, one adds, deletes or modifies lines
 * of a configuration file (its location is defined in iptrace.h).
 * The running iptrace reconfigures from this file when signalled
 * with SIGHUP
 *
 * By:  A. Mankin
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */

#include "iptrace.h"


/*
 * Variable for debugging.
 */
int verbose = VERBOSE;

/*
 * Initialize global file pointers.
 */
FILE *TRACE = NULL, *CONF = NULL, *HTM = NULL, *SAMPLE = NULL;

/*
 * Other globals.
 */
char datestamp[26];
int needconfig = 0;

/*
 * Handler of SIGUSR1;
 */
config()
{
  needconfig = 1;
}


/*
 * Configuration called by main at startup and at
 * some point after SIGUSR1 received.
 */
doconfig(sig)
     int sig;
{
    int i, tokens;
    static char *configline[MAX_CONFIG_TOKENS];

    /*
     * Parameters set in configuration file.
     */
    int htm_mins;

    
    makedatestamp(datestamp);

    if(verbose)
      fprintf(stderr, "configuring iptrace at %s\n", datestamp);


    if ( (CONF = fopen(IPTRACE_CONF_FILE, "r")) == NULL) {
      if(verbose)
        fprintf("config file not found\n", IPTRACE_CONF_FILE);
      exit(-1);
    }

    if (sig == 0)
      for (i = 0; i < MAX_CONFIG_TOKENS; i++)
	configline[i] = (char *)malloc((unsigned)CONFIG_TOKEN_SIZE); 

    while ((tokens = getConfig(configline)) != 0) {

      if (strcmp(configline[0], "outfile") == 0) 
	dofile(tokens, configline);

      else if (strcmp(configline[0], "nofile") == 0) 
	stopfile(tokens, configline);

      else if (strcmp(configline[0], "set") == 0) {
	if (tokens != 3) {
	  if (verbose)
	    fprintf(stderr, "config: bad set line\n");
	  continue;
	}
	if(strcmp (configline[1], "htminterval") == 0) 
	  htm_mins = atoi(configline[2]);

	else if (verbose)
	  fprintf(stderr, "config: can't set %s\n", configline[1]);
      }
      else if (verbose)
	fprintf(stderr, "config: can't configure %s\n", configline[0]);

    }

    fclose(CONF);

    return;
}

dofile(argc, args) 
     int argc;
     char **args;
{
  if(argc != 3)
    return;

  if(strcmp(args[1], "data") == 0) {
    if(TRACE) {
      fflush (TRACE);
      fclose(TRACE);
    }
    TRACE = fopen(args[2], "a");
    if (TRACE == NULL) 
     fprintf(stderr, "config: couldn't open requested tracefile %s at %s\n",
	     args[2], datestamp);
    else
      fprintf(TRACE, "%s\n", datestamp);
  }

  else if(strcmp(args[1], "htm") == 0) {
    if (HTM) {
      fflush(HTM);
      fclose(HTM);
    }
    HTM = fopen(args[2], "a");
    if (HTM == NULL) 
     fprintf(stderr, "config: couldn't open requested htmfile %s at %s\n",
	     args[2], datestamp);
    else
      fprintf(HTM, "%s\n", datestamp);
  }

  else if(strcmp(args[1], "sample") == 0) {
    if (SAMPLE) {
      fflush(SAMPLE);
      fclose(SAMPLE);
    }
    SAMPLE = fopen(args[2], "a");
    if (SAMPLE == NULL) 
     fprintf(stderr, "config: couldn't open requested samplefile %s at %s\n",
	     args[2], datestamp);
    else
     fprintf(SAMPLE, "%s\n", datestamp);
  }

  else {
    if(verbose)
      fprintf(stderr, "config: bad file type\n");
    return;
  }
  
  if(verbose)
    fprintf(stderr, "config: %sfile %s at %s\n",
	    args[1], args[2], datestamp);
  
}

stopfile(argc, args) 
     int argc;
     char *args[];
{

  if(argc != 2)
    return;

  if(strcmp(args[1], "data") == 0) {
    if(TRACE) {
      fflush(TRACE);
      fclose(TRACE);
      if(verbose)
	fprintf(stderr, "config: closed %sfile %s at %s\n",
		args[1], args[2], datestamp);
    }
    TRACE = NULL;
  }

  else if(HTM && (strcmp(args[1], "htm") == 0)) {
    if(HTM) {
      fflush(HTM);
      fclose(HTM);
    }
    HTM = NULL;
  }

  else if(SAMPLE && (strcmp(args[1], "sample") == 0)) {
    if(SAMPLE) {
      fflush(SAMPLE);
      fclose(SAMPLE);
    }
    SAMPLE = NULL;
  }

  else {
    if(verbose)
      fprintf(stderr, "config: bad file type\n");
    return;
  }

}

getConfig(configv)
     char *configv[];
{
  char c;
  int i, building, numtokens;
  char token[CONFIG_TOKEN_SIZE], *cp;


  i = building = numtokens = 0;
  cp = token;

  while((c = getc(CONF)) != EOF) {

    printf("\n%c",c);
    switch (c) {
    case '\t':
    case ' ':
      if (!building)
	continue;
      else {
	numtokens++;
	*cp = 0;
	printf("\ttoken #%d = %s\n", i, token);
	strcpy(configv[i], token);
	cp = token;
	building = 0;
	i++;
      }
      printf(" building=%d numtokens=%d \n", building, numtokens, i);
      break;

    case '\n':
      if (building)
	numtokens++;
      printf(" building=%d numtokens=%d \n", building, numtokens, i);
      *cp = 0;
      printf("\ttoken=%s\n", token);
      strcpy(configv[i], token);
      return(numtokens);

    case '#':
      while((c = getc(CONF)) != '\n');
      break;

    default:
      building = 1;
      *cp++ = c;
      printf(" building=%d numtokens=%d \n", building, numtokens, i);
      break;
    }
  }

  return (0);
}


/*
 * Called when SIGTERM received.
 */
cleanup()
{
     if(TRACE) {
       host_matrix(TRACE);
       fflush(TRACE);
     }
     if(HTM)
       fflush(HTM);
     if(SAMPLE)
       fflush(SAMPLE);
     exit(0);
}

/*
 * Called when SIGINT received.
 */
dosumm()
{
  if (SAMPLE)
    host_matrix(SAMPLE);
  return;
}

/*
 * Called by:  interval timer (SIGALRM)
 */
dotimer()
{

    host_matrix(HTM);

    return;
}

timerset(mins) 
{
    struct itimerval interv;    

    bzero(&interv, sizeof interv);
    interv.it_interval.tv_sec = interv.it_value.tv_sec = mins * 60;

    setitimer(ITIMER_REAL, &interv, (struct itimerval *)0);

}

timerunset() {
     struct timeval interv;

     bzero(&interv, sizeof interv);
     setitimer(ITIMER_REAL, &interv, (struct itimerval *)0);
}


makedatestamp(str) 
     char str[];
{
    struct timeval tv;

    gettimeofday(&tv, (struct timeval *)0);
    strcpy((char *)str, (char *)ctime(&tv));
    /*
     * Get rid of embedded newline.
     */
    str[24] = 0;

}
