/*	netmon.c  NETMON (Mitre) 	8/31/88		*/

/*
 * netmon.c:  UNIX kernel network protocol logging.
 * By: A. Mankin
 *
 *  Double buffer/fixed-size records/little interrupt disabling.
 *
 * Copyright (c) 1989, the MITRE Corporation.  All rights reserved.
 * This software was developed by the MITRE Corporation, McLean VA.
 * It is distributed free of charge with the following conditions:
 *
 *      1.  Redistribution and use in source and binary forms are permitted
 *	    provided that this copyright notice be duplicated in all such
 *	    forms.
 *
 *      2.  MITRE will be credited should the software be used in a
 *          product or written about in any publications.  However,
 *          MITRE will not be referenced as the source in advertisements.
 *
 *      3.  MITRE assumes no legal responsibility for source code and
 *          its subsequent use.  No warranty is expressed or implied.
 *
 *      4.  If any bugs or problems are found, then they should be
 *          reported back to MITRE.
 */
#include "param.h"
#include "systm.h"
#include "mbuf.h"
#include "errno.h"
#include "time.h"
#include "kernel.h"
#include "file.h"
#include "uio.h"
#include "ioctl.h"

#define NETMON_KERNEL
#include "netmon.h"

#define NM_BUFRECS	(NM_MAXTRACED >> 1) 	/* Half in each buffer */

#define NM_WRITE	FWRITE
#define NM_READ		FREAD

struct netmonbuf {
     short rw;
     short off;
     struct netmonstr recs[NM_BUFRECS];
} netmonbuf[2];

struct netmoncb  {
     short currbuf;
     short readwait;
     short readminoff;
     short sigminoff;
     u_long drops;
} netmoncb;

int netmon_open = 0;	/* Enforces one NETMON user */
int netmonp = 0;	/* Flag tested by protocol  */
int netmonseq = 0;
int netmonincr = 1;

/*
 * Called by: nmopen which is called through cdevsw by open syscall.
 */
nm_init() 
{

     /*
      * Initialize control block.
      */
     netmoncb.drops = 0;
     netmoncb.readminoff = NM_BUFRECS - (NM_BUFRECS >> 2);	
     netmoncb.sigminoff = NM_BUFRECS - (NM_BUFRECS >> 3);

     /*
      * NETMON will write first in netmonbuf[0];
      */
     netmoncb.currbuf = 0;	

     netmoncb.readwait = 0;

     /*
      * Initialize both buffers.
      */
     bzero((char *)netmonbuf, sizeof(struct netmonbuf));
     bzero((char *)(netmonbuf + 1), sizeof(struct netmonbuf));
     netmonbuf[0].rw = netmonbuf[1].rw = NM_WRITE;

     /*
      * Turn on monitoring.
      */
     printf("netmon init\n");
     netmonp = 1;
     return(0);
}


/*
 * Called by: IP and driver i/o routines.
 */
doNetmon(m, loc, code, info, ql, copydata) 
     struct mbuf *m;
     short loc, code, ql, copydata;
     u_long info;
{ 

     int Buf, Record; 
     register struct netmonstr *nt; 
     int s;
     
     /*
      * Address of m is needed whether or not this is main record;
      * used for matching associated records with their main.
      */
     if(m == 0) {
	  printf("netmon: bad probe 1\n");
	  return;
     }

     if(copydata && m->m_len == 0) {
	  printf("netmon: bad probe 2\n");
	  return;
     }

     /*
      * Disable writes (i.e. net interrupts) just long enough to
      * grab a blank record.
      */
     s = splimp();

     /*
      * If current buffer full, make it readable and switch 
      * to other buffer.
      */

     if(netmonbuf[netmoncb.currbuf].off + netmonincr > NM_BUFRECS) {

	  netmonbuf[netmoncb.currbuf].rw = NM_READ;

	  netmoncb.currbuf ^= 1;

	  /*
	   * If process hasn't read switched-to buffer
	   * (i.e. if it's marked readable), then must 
	   * drop contents, make it writable.
	   */
	  if(netmonbuf[netmoncb.currbuf].rw == NM_READ) {

	       netmoncb.drops += NM_BUFRECS;

	       netmonbuf[netmoncb.currbuf].rw = NM_WRITE;

	       netmonstat.fullbufferdrops++;
	  }

          netmonbuf[netmoncb.currbuf].off = 0;

	  netmonstat.fullbufferflips++;

     }

     /*
      * Usually just these three lines are executed 
      * with interrupts disabled.
      */
     Buf = netmoncb.currbuf;
     Record = netmonbuf[Buf].off;
     netmonbuf[Buf].off += netmonincr;

     /*
      * Reenable network interrupts.
      */
     splx(s);

	  
     /*
      * Make the record.
      */
     nt = &(netmonbuf[Buf].recs[Record]);

     nt->seqno = netmonseq++;

     microtime(&nt->ts); 

     nt->ql = (u_char)ql; 
     nt->loc = loc;
     nt->code = code;
     nt->info = info;
     nt->id = (caddr_t)m;

     if(copydata) 
	  nt->len = netmon_mcopy(m, nt->data);
     else
	  nt->len = 0;

     netmonstat.records++;

     /*
      * If reading process is asleep and enough records now
      * in buffer, wake it.
      */
     if(netmoncb.readwait && (Record >= netmoncb.readminoff) ) {

	  netmoncb.readwait = 0;

	  netmonstat.readwakes++;

	  wakeup(netmonbuf);

     }

     return(0);
} 

/*
 * Called by: open system call through cdevsw.
 */
nmopen(dev, mode)
     dev_t dev;
     int mode;
{
     if((mode & (FREAD|FEXCL) == 0) || minor(dev))
	  return(EINVAL);

     if(netmon_open)
	  return(EBUSY);

     netmon_open = 1;

     (void)nm_init();

     return(0);
}

/*
 * Called by: close system call through cdevsw.
 */
nmclose(dev, flag)
     dev_t dev;
     int flag;
{
     netmonp = 0;
     netmon_open = 0;

     printf("netmon off\n");

     return(0);
}

/*
 * Called by: read system call through cdevsw.
 */
nmread(dev, uio)
     dev_t dev;
     struct uio *uio;
{
     register Readbuf;
     register long len;
     register caddr_t bufptr;
     int error = 0;
     int s, i;



     /*
      * If too few records in buffer, put reading process to
      * sleep.  Wakeup is in doNetmon.
      */

     if(netmonbuf[netmoncb.currbuf].off < netmoncb.readminoff) {

	  netmoncb.readwait++;

	  netmonstat.readsleeps++;

	  while (netmoncb.readwait)
	       sleep(netmonbuf, PZERO + 1);	/* wakeable by signal */

     }

     /*
      * Process may read.  Disable writes (i.e. net interrupts)
      * just long enough to switch writing to other buffer.
      */
     s = splimp();

     Readbuf = netmoncb.currbuf;
     netmonbuf[Readbuf].rw = NM_READ;

     netmoncb.currbuf ^= 1;

     netmonstat.readbufferflips++;

     /*
      * If process hasn't read switched-to buffer, drop 
      * the contents and make it writable.
      */
     if(netmonbuf[netmoncb.currbuf].rw == NM_READ) {

	  netmoncb.drops += netmonbuf[netmoncb.currbuf].off;
	  netmonbuf[netmoncb.currbuf].rw = NM_WRITE;
	  netmonbuf[netmoncb.currbuf].off = 0;

	  netmonstat.fullbufferdrops++;

     }

     /*
      * Reenable network interrupts.
      */

     splx(s);

     /*
      * Copy the NETMON buffer to user-level buffer.
      */

     len = netmonbuf[Readbuf].off * (sizeof(struct netmonstr));

     bufptr = (char *)&(netmonbuf[Readbuf].recs[0]);

     len = MIN(len, uio->uio_resid);

     if (len == 0)
	  return(EINVAL);

     error = uiomove(bufptr, (int)len, UIO_READ, uio);

     netmonbuf[Readbuf].off = 0;
     netmonbuf[Readbuf].rw = NM_WRITE;

     return (error);
}


/*
 * Called by:  doNetmon.
 *
 * Copy NM_MAXDATA bytes from an mbuf chain.
 * Returns the number of bytes successfully copied.
 */
netmon_mcopy(m, cp)
     register struct mbuf *m;
     register char *cp;
{
     register int len;
     register int ml;

     if (m->m_len == 0)
	  return (0);

     len = NM_MAXDATA;
     while (len && m) {
	  ml = MIN(len, m->m_len);
	  bcopy(mtod(m, char *), cp, ml);
	  cp += ml;
	  len -= ml;
	  m = m->m_next;
     }
     
     return (NM_MAXDATA - len);
}
